import pytest

from pyramid_secure_response.ssl_redirect import tween


@pytest.fixture(autouse=True)
def setup():
    import logging
    from pyramid_secure_response.ssl_redirect import logger
    logger.setLevel(logging.ERROR)


def test_redirect_tween_ssl_redirect_off(mocker, dummy_request):
    mocker.patch('pyramid_secure_response.ssl_redirect.apply_ignore_filter',
                 return_value=True)
    mocker.patch('pyramid_secure_response.ssl_redirect.build_criteria',
                 return_value=[])

    from pyramid_secure_response.ssl_redirect import (
        apply_ignore_filter,
        build_criteria,
    )

    dummy_request.registry.settings = {
        'pyramid_secure_response.ssl_redirect': 'False'
    }

    handler_stub = mocker.stub(name='handler_stub')
    ssl_redirect_tween = tween(handler_stub, dummy_request.registry)
    ssl_redirect_tween(dummy_request)

    # pylint: disable=no-member
    assert 1 == handler_stub.call_count
    assert 0 == apply_ignore_filter.call_count
    assert 0 == build_criteria.call_count


def test_redirect_tween_ignored_path(mocker, dummy_request):
    mocker.patch('pyramid_secure_response.ssl_redirect.apply_ignore_filter',
                 return_value=True)
    mocker.patch('pyramid_secure_response.ssl_redirect.build_criteria',
                 return_value=[])

    from pyramid_secure_response.ssl_redirect import (
        apply_ignore_filter,
        build_criteria,
    )

    dummy_request.path = '/humans.txt'
    dummy_request.registry.settings = {
        'pyramid_secure_response.ssl_redirect': 'True',
        'pyramid_secure_response.ignore_paths': '\n/humans.txt\n'
    }

    handler_stub = mocker.stub(name='handler_stub')
    ssl_redirect_tween = tween(handler_stub, dummy_request.registry)
    ssl_redirect_tween(dummy_request)

    # pylint: disable=no-member
    assert 1 == handler_stub.call_count
    assert 1 == apply_ignore_filter.call_count
    apply_ignore_filter.assert_called_once_with(
        dummy_request, ('/humans.txt',))
    assert 0 == build_criteria.call_count


def test_redirect_tween_insecure(mocker, dummy_request):
    mocker.patch('pyramid_secure_response.ssl_redirect.apply_ignore_filter',
                 return_value=False)
    mocker.patch('pyramid_secure_response.ssl_redirect.build_criteria',
                 return_value=[False])

    from pyramid.httpexceptions import HTTPMovedPermanently

    from pyramid_secure_response.ssl_redirect import (
        apply_ignore_filter,
        build_criteria,
    )

    dummy_request.url = 'http://example.org/'
    dummy_request.registry.settings = {
        'pyramid_secure_response.ssl_redirect': 'True',
    }

    handler_stub = mocker.stub(name='handler_stub')
    ssl_redirect_tween = tween(handler_stub, dummy_request.registry)

    with pytest.raises(HTTPMovedPermanently):
        ssl_redirect_tween(dummy_request)

    # pylint: disable=no-member
    assert 0 == handler_stub.call_count
    assert 1 == apply_ignore_filter.call_count
    apply_ignore_filter.assert_called_once_with(dummy_request, tuple())
    assert 1 == build_criteria.call_count


def test_redirect_tween_secure(mocker, dummy_request):
    mocker.patch('pyramid_secure_response.ssl_redirect.apply_ignore_filter',
                 return_value=False)
    mocker.patch('pyramid_secure_response.ssl_redirect.build_criteria',
                 return_value=[True])

    from pyramid_secure_response.ssl_redirect import (
        apply_ignore_filter,
        build_criteria,
    )

    dummy_request.url = 'https://example.org/'
    dummy_request.registry.settings = {
        'pyramid_secure_response.ssl_redirect': 'True',
    }

    handler_stub = mocker.stub(name='handler_stub')
    ssl_redirect_tween = tween(handler_stub, dummy_request.registry)
    ssl_redirect_tween(dummy_request)

    # pylint: disable=no-member
    assert 1 == handler_stub.call_count
    assert 1 == apply_ignore_filter.call_count
    apply_ignore_filter.assert_called_once_with(dummy_request, tuple())
    assert 1 == build_criteria.call_count



from collections import namedtuple
import logging

from pyramid_secure_response import __name__ as PACKAGE_NAME


logger = logging.getLogger(PACKAGE_NAME)  # pylint: disable=invalid-name


def get_config_value_fn(registry, config_key=PACKAGE_NAME):
    # type: (Registry, str) -> 'function'
    """Gets configured value from .ini file via registry."""
    if not config_key:
        raise ValueError

    s = registry.settings

    def _get_value(key, default):
        v = s.get('{:s}.{:s}'.format(config_key, key), default)
        if v == default:
            return v
        v = str(v)
        if v.lower() == 'true':
            return True
        elif v.lower() == 'false':
            return False
        if '\n' in v:
            return tuple(v.split())

        return v

    return _get_value


def get_config(registry):  # type: (Registry) -> namedtuple
    """Returns namedtuple instance object has configuration."""
    get_value = get_config_value_fn(registry)

    defaults = [
        ('hsts_support', True),
        ('ssl_redirect', True),

        ('proto_header', ''),   # e.g. X-Forwarded-Proto
        ('ignore_paths', tuple()),

        # HSTS Header value
        ('hsts_max_age', '31536000'),  # seconds, 1 year
        ('hsts_include_subdomains', True),
        ('hsts_preload', True),
    ]

    # pylint: disable=invalid-name
    Config = namedtuple('Config', [k for k, _ in defaults])
    return Config(**dict([(k, get_value(k, v)) for k, v in defaults]))


def apply_ignore_filter(req, ignore_paths):  # type: (Request, tuple) -> bool
    if ignore_paths:
        return any([req.path.startswith(path) for path in ignore_paths])
    return False


def build_criteria(req, config):  # type: (Request, namedtuple) -> tuple
    """Builds criteria contains about incoming request."""
    criteria = [
        req.url.startswith('https://'),
    ]
    if config.proto_header:
        criteria.append(
            req.headers.get(config.proto_header, 'http') == 'https')

    return tuple(criteria)



import os

from setuptools import setup, find_packages


# pylint: disable=invalid-name
here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(here, *('doc', 'DESCRIPTION.rst'))) as f:
        DESCRIPTION = f.read()
    with open(os.path.join(here, 'CHANGELOG')) as f:
        CHANGELOG = f.read()
except IOError:
    DESCRIPTION = ''
    CHANGELOG = ''


def file_path(name):
    return os.path.join(os.path.dirname(__file__), name)


def read_file(path):
    return open(path, 'r').read()


def get_attr(name, attr):
    import re

    pattern = r"{0}\s=\s'([^']+)'".format(attr)
    value, = re.findall(pattern, read_file(file_path(name)))
    return value


requires = [
    'pyramid',
]

development_requires = [
    'flake8',
    'flake8_docstrings',
    'pycodestyle',
    'pylint',
]

testing_requires = [
    'pytest',
    'pytest-cov',
    'pytest-mock',
]

documentation_requires = [
]

setup(
    name='pyramid_secure_response',
    version=get_attr('pyramid_secure_response/__init__.py', '__version__'),
    description='A library redirects non-http as https, sets hsts header.',
    long_description=DESCRIPTION + '\n\n' + CHANGELOG,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Pyramid',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Security',
    ],
    author='Yasuhiro Asaka',
    author_email='yasuhiro.asaka@grauwoelfchen.net',
    url='https://gitlab.com/grauwoelfchen/pyramid_secure_response',
    keywords='web wsgi pylons pyramid',
    license='BSD-3-Clause',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    extras_require={
        'development': development_requires,
        'testing': testing_requires,
        'documentation': documentation_requires,
    },
    install_requires=requires,
    entry_points='',
)



import pytest

from pyramid_secure_response.util import (
    get_config,
    apply_ignore_filter,
    build_criteria,
)


def test_get_config_keys(dummy_request):
    config = get_config(dummy_request.registry)
    expected_keys = (
        'hsts_support',
        'ssl_redirect',
        'proto_header',
        'ignore_paths',
        'hsts_max_age',
        'hsts_include_subdomains',
        'hsts_preload',
    )
    assert expected_keys == tuple(config._asdict().keys())


@pytest.mark.parametrize('config_key,default_value', [
    ('hsts_support', True),
    ('ssl_redirect', True),
    ('proto_header', ''),
    ('ignore_paths', tuple()),
])
def test_get_config_defaults(dummy_request, config_key, default_value):
    config = get_config(dummy_request.registry)
    assert default_value == getattr(config, config_key)


@pytest.mark.parametrize('paths', [
    tuple(),
    ('/foo', '/bar'),
    ('humans.txt', 'robots.txt'),
    ('/static/humans.txt',),
])
def test_apply_ignore_filter_as_not_ignored(dummy_request, paths):
    dummy_request.path = '/humans.txt'
    assert not apply_ignore_filter(dummy_request, paths)


@pytest.mark.parametrize('paths', [
    ('/humans.txt',),
    ('/humans.txt', '/robots.txt'),
])
def test_apply_ignore_filter_as_ignored(dummy_request, paths):
    dummy_request.path = '/humans.txt'
    assert apply_ignore_filter(dummy_request, paths)


@pytest.mark.parametrize('url,proto_header,header,value', [
    ('http://127.0.0.1/', '', None, None),
    ('http://127.0.0.1/', '', 'X-Forwarded-Proto', 'http'),
    ('http://127.0.0.1/', 'X-Forwarded-Proto', 'X-Forwarded-Proto', None),
    ('http://127.0.0.1/', 'X-Forwarded-Proto', 'X-Forwarded-Proto', 'http'),
    ('https://127.0.0.1/', 'X-Forwarded-Proto', 'X-Forwarded-Proto', None),
    ('https://127.0.0.1/', 'X-Forwarded-Proto', 'X-Forwarded-Proto', 'http'),
])
def test_criteria_not_all_secure(dummy_request, url, proto_header,
                                 header, value):
    from collections import namedtuple

    dummy_request.url = url
    dummy_request.headers[header] = value

    # pylint: disable=invalid-name
    Config = namedtuple('Config', 'proto_header')
    config = Config(proto_header)

    criteria = build_criteria(dummy_request, config)
    assert not all(criteria)


@pytest.mark.parametrize('url,proto_header,header,value', [
    ('https://127.0.0.1/', '', None, None),
    ('https://127.0.0.1/', '', 'X-Forwarded-Proto', 'https'),
    ('https://127.0.0.1/', 'X-Forwarded-Proto', 'X-Forwarded-Proto', 'https'),
])
def test_criteria_all_secure(dummy_request, url, proto_header,
                             header, value):
    from collections import namedtuple

    dummy_request.url = url
    dummy_request.headers[header] = value

    # pylint: disable=invalid-name
    Config = namedtuple('Config', 'proto_header')
    config = Config(proto_header)

    criteria = build_criteria(dummy_request, config)
    assert all(criteria)
